#
# Common utilities used by TSIPP example scripts.
#
# $Id: utils.tcl,v 1.5 1997/07/20 21:42:28 markd Exp $
#

#
# Command to view images.  Change this to your favorite viewer if you don't
# have xv.  If you have a 24 bit display, remove the -8 and -owncmap
# argruments.
#
set VIEWER "| xv -8 -owncmap -"

#
# Fatal --
#
#  Print a message to stderr and exit.
#
proc Fatal args {
    foreach msg $args {
        puts -nonewline stderr $msg
    }
    puts stderr ""
    exit 1
}

#
# ParseArgs --
#
# Parse the command line arguments and set global variables that control the
# generation of images.  The globals that are set are the same as the argument
# (less the `-').  All values are defaulted.  The following arguments are
# supported:
#      o -output file - Output to file instead of a viewer.
#      o -size size - Generate an image of size X size pixels.
#      o -mode mode - The rendering mode.
#      o -backfaces - Show backfaces if specified.
#      o -final - Generate a final version of an image.
#      o -texture texture - Use the specified texture mapping.
#      o -misc string - Any miscellaneous argument.
#
proc ParseArgsUsage {} {
    global argv0
    Fatal  "usage: $argv0 " {-output file -size size } \
            {-mode PHONG|GOURAUD|FLAT|LINE -backfaces -final } \
            {-texture texture -misc string}
}

proc GetFlagValue {argListVar flagName} {
    upvar $argListVar argList
    set value [lvarpop argList]
    if [lempty $value] {
        Fatal "missing argument to $flagName"
    }
    return $value
}

proc ParseArgs {} {
    global argv VIEWER 
    global output size mode backfaces final texture misc

    set output $VIEWER
    set size 256
    set mode PHONG
    set backfaces 0
    set final 0
    set texture WORLD
    set misc {}

    set argList $argv
    while {![lempty $argList]} {
        set arg [lvarpop argList]
        switch -- $arg {
            {-output} {
                set output [GetFlagValue argList $arg]
            }
            {-size} {
                set size [GetFlagValue argList $arg]
            }
            {-mode} {
                set mode [GetFlagValue argList $arg]
            }
            {-backfaces} {
                set backfaces 1
            }
            {-final} {
                set final 1
            }
            {-texture} {
                set texture [GetFlagValue argList $arg]
            }
            {-misc} {
                set misc [GetFlagValue argList $arg]
            }
            default {
                puts stderr "Unknown argument \"$arg\":"
                ParseArgsUsage
            }
        }
    }

    set mode [string toupper $mode]
    if {[lsearch {PHONG GOURAUD FLAT LINE} $mode] < 0} {
        Fatal "invalid mode \"$mode\" " {expected one of "PHONG", } \
                {"GOURAUD", "FLAT", or "LINE"}
    }
    set texture [string toupper $texture]
    if {[lsearch {NATURAL CYLINDRICAL SPHERICAL WORLD} $texture] < 0} {
        Fatal "invalid texture \"$texture\" " {expected one of } \
                {"NATURAL", "CYLINDRICAL", "SPHERICAL", or "WORLD"}
    }
    # White background, as it looks better in print.
    SippBackground {1.0 1.0 1.0}
}

#
# Standard light source.  Optionally takes location and color as arguments.
#
proc StdLights {{location {15.0 15.0 15.0}} {color {0.9 0.9 0.9}}} {
    SippLightSourceCreate $location $color POINT
}

#
# Standard rendering.  Optionally takes camera point, camera look at and focal
# ratio as arguments.
#
proc StdRender {{cameraPoint {-2.0 2.0 5.0}}  {cameraLook {0.0 0.0 0.0}}
                {focal 0.2}} {
    global backfaces output size mode final

    SippCameraParams STDCAMERA $cameraPoint $cameraLook {0.0 1.0 0.0} $focal
    SippShowBackFaces $backfaces

    set pbm [SippPBMOpen $output w]
    SippRender $pbm $size $size $mode [expr $final ? 3 : 1]
    SippPBMClose $pbm
}
